/*
 *  make IO for Yarn -  rg070596
 *
 *  Please read the following sections carefully, because there is really no
 *  other help available (at the moment)
 *
 *  rg130696:
 *  - now using VSoup for news reception
 *  - big update:
 *    simultaneous mail/news reception/transmission
 *  rg180796:
 *  - 'import' retries in an endlos loop until soup.zip has been accepted
 *  - if SEND fails, the messages are zipped again, which means they can be edited
 *    again in Yarn
 *  rg080996:
 *  - auto connect/disconnect, if required
 *  - user configuration section better indicated
 *  - YarnBinDir no longer required (see config section)
 *  - the secondary newsserver can be defined via YARNIONEWS2 environment variable
 *  - if YARNIONEWS2 is empty, the secondary news server is disabled by default
 *  - yarnio-operation a little bit more secure (chance of packet loss minimized...)
 *  - new command line options for disabling specific services
 *  rg130996:
 *  - semaphor for each service
 *  - legal to start multiple instances of YarnIo
 *  - simple signal handling for logoutISP (signal handling is real b*llsh*t in REXX,
 *    because one has to use the environment vars)
 *  - YarnIo is checking %YARN%\history.pag before starting import...
 *  rg170996:
 *  - configuration file yarnio.set (resides in the same directory as yarnio.cmd)
 *    now user configuration and YarnIo are independent
 *  rg200996:
 *  - zipProg/unzipProg now in YarnIo.Set
 *  rg220996:
 *  - signal handling removed!  To check and abort the connection an extra CMD
 *    will be opened
 *
 *  environment vars:
 *  - home
 *  - tmp
 *  - yarn
 *
 *  directory structure (must be set up manually!):
 *  - %home%\yarn\in       - files for reception
 *  - %home%\yarn\in\mail  - files for received mail
 *  - %home%\yarn\in\news  - files for received news
 *  - %home%\yarn\in\news2 - files for received news from alternative source
 *  - %home%\yarn\out      - souper packets for transmission
 *  - %yarn%               - yarn subdir (history.pag required)
 *
 *  external utilities (configured in yarnio.set):
 *  - vsoup.exe        - soup packet receiver transmitter
 *  - import.exe       - import messages to Yarn database
 *  - zip              - zip files
 *  - unzip            - unzip files
 *  - ping             - ping a host (exit status 0 indicates there is a connection)
 *  - loginisp.cmd     - connect to your ISP
 *  - logoutisp.cmd    - disconnect from your ISP
 *                       if you have an auto-dialer, loginisp/logoutisp can be empty cmd files
 *
 *  options:
 *  -SEND      - no mail/news transmission
 *  -RCVMAIL   - no mail reception
 *  -RCVNEWS   - no news reception form primary newsserver
 *  -RCVNEWS2  - no news reception form secondary newsserver
 *
 *  internal options:
 *   SEND      - start the send process
 *   RCVMAIL   - start the receive mail process
 *   RCVNEWS   - start the receive news process
 *   RCVNEWS2  - start the alternate receive news process
 *   CHECKISP  - if YarnIo failes, connection will be killed
 *  with no option given, YarnIo starts all the above processes
 *
 *  todo:
 *  - restructure script, so that there are several procedures (& stems)
 *  - handle more than one single news server (in a clean way)
 */

call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
call SysLoadFuncs

DEBUG = 0

/*
 *  get some global vars
 */
env = 'os2environment'
homeDir = value('home',,env)
tmpDir  = value('tmp',,env)
yarnDir = value('yarn',,env)
parse source . . compCmdName
cmdName = filespec('name',compCmdName)
CompSetName = delstr(CompCmdName,lastpos('.',CompCmdName)) || '.set'

if (homeDir = '')  |  (tmpDir = '')  |  (yarnDir = '') then
    call Help 'Either HOME, TMP or YARN not defined in environment'

/*********************************************************************************
 *********************************************************************************
 *
 *  begin of user configuration section
 *
 *  define some global vars (commands / directories)
 *  change, if you like to
 *
 *  NEVERTHELESS IT IS NOT RECOMMENDED TO CHANGE ANYTHING IN THIS SECTION
 *       TRY TO KEEP YOUR KONFIGURATION IN THE YARNIO.SET FILE
 */

xmtDir        = homeDir || '\yarn\out'
soupXmtZip    = 'reply.zip'                 /* same as in yarn-config! */
rcvDir        = homeDir || '\yarn\in'
rcvNewsDir    = rcvDir  || '\news'
rcvNews2Dir   = rcvDir  || '\news2'
rcvMailDir    = rcvDir  || '\mail'

/*
 *  set up some configuration defaults
 */
soupSend      = ''
soupRcvNews   = ''
soupRcvNews2  = ''
soupRcvMail   = ''
echoProlog    = 'echo' CompSetName || ': no'
importProg    = echoProlog 'importProg defined:'
pingHost      = echoProlog 'pingHost defined'
connectISP    = echoProlog 'connectISP defined'
hangupISP     = echoProlog 'hangupISP defined'
unzipProg     = echoProlog 'unzipProg defined'
zipProg       = echoProlog 'zipProg defined'

/*
 *  the following could be changed, but this is not required (at least if you have zip/unzip)
 */
soupRcvZip    = 'soup.zip'
yarnHistory   = yarnDir || '\history.pag'
soupRcvAreas  = 'areas'
SoupRcvMsgs   = '*.msg'
SoupXmtReply  = 'replies'
SoupXmtMsgs   = '????.msg'

/*
 *  interprete the user configuration
 */
if \FileExists(CompSetName) then do
   say CompSetName 'required for configuration'
   exit 3
end
'@type' CompSetName '| RXQUEUE'
do ii = 1 to queued()
   parse pull line
   if substr(strip(line),1,1) \= '#' then
      interpret line
end

unzipProg     = unzipProg soupXmtZip
zipProgTmp    = zipProg
zipProg       = zipProgTmp soupRcvZip soupRcvAreas soupRcvMsgs
zipProgSend   = zipProgTmp soupXmtZip soupXmtReply soupXmtMsgs

/*
 *  end of configuration section
 *
 *********************************************************************************
 *********************************************************************************/

rc = setlocal()
'@set EMXOPT=-h40 -c -n'   /*  set the maximum number of handles to 40  */
'@set nntpserver='         /* clear the NNTPSERVER envorinment variable */

/*
 *  files for internal use
 */
semActive   = tmpdir  || '\yarnio.act'
semImport   = tmpdir  || '\yarnio.imp'
flgRcvNews  = tmpdir  || '\yarnio.frn'
flgRcvNews2 = tmpDir  || '\yarnio.fr2'
flgRcvMail  = tmpDir  || '\yarnio.frm'
flgXmt      = tmpDir  || '\yarnio.fx'

if \DEBUG then do
   startProc = 'start /c /b /min'
end
else do
   startProc = 'start /c /b'
end

if DEBUG then 
   trace('?i')

/*
 *  get command line options
 */
Option = ''
AddOption = ''
parse upper arg ropt
do while ropt \= ''
   parse var ropt opt ropt
   add = 1
   select
      when opt = '-RCVNEWS' then
         soupRcvNews = ''
      when opt = '-RCVNEWS2' then
         soupRcvNews2 = ''
      when opt = '-RCVMAIL' then
         soupRcvMail = ''
      when opt = '-SEND' then
         soupSend = ''
      otherwise do
         Option = opt
         add = 0
      end
   end
   if add then
      AddOption = AddOption opt
end

/*
 *  do the thing
 */
if Option = '' then do
   wasConnected = 1;
   haveSema = 0;
   if stream(semActive,'c','open write') = 'READY:' then do
      haveSema = 1;
      wasConnected = Connect()
   end
   /*
    *  initiate all other processes and wait for completion
    */
   StartTime = time('S')
   say 'starting the transmitting/receiving processes...'
   rc = DeleteFile( flgRcvNews )    /* fails, if already started... */
   rc = DeleteFile( flgRcvNews2 )
   rc = DeleteFile( flgRcvMail )
   rc = DeleteFile( flgXmt )
   '@' startProc '"YarnIo SEND"'     CompCmdName 'SEND'     AddOption
   '@' startProc '"YarnIo RCVMAIL"'  CompCmdName 'RCVMAIL'  AddOption
   '@' startProc '"YarnIo RCVNEWS"'  CompCmdName 'RCVNEWS'  AddOption
   '@' startProc '"YarnIo RCVNEWS2"' CompCmdName 'RCVNEWS2' AddOption
   State = 'aborted'
   if haveSema then do
      say 'waiting...'
      say ''
      rc = SysSleep( 10 )     /*  time to start subprocesses  */
      loopCnt = 0
      do forever
         rc = SysSleep( 1 )
         loopCnt = loopCnt + 2
         if loopCnt > 3600 then do
            say ''
            say 'timeout:  cleaning up and exit...'
            leave
         end
         rc = CheckFlag( flgXmt,     'SEND:    ' );
         rc = CheckFlag( flgRcvMail, 'RCVMAIL: ' );
         rc = CheckFlag( flgRcvNews, 'RCVNEWS: ' );
         rc = CheckFlag( flgRcvNews2,'RCVNEWS2:' );
         rc = SysSleep( 1 )
         if \FileExists(flgRcvNews) & \FileExists(flgRcvNews2) & \FileExists(flgRcvMail) & \FileExists(flgXmt) then do
            say ''
            say 'finished...'
            leave
         end
      end
      State = 'ok'
   end
   else
      State = 'already active'
   rc = CleanUp( State )
end
else if Option = 'CHECKISP' then do
   say 'checking the connection...'
   do forever
      rc = SysSleep( 5 )
      if stream(semActive,'c','open write') = 'READY:' then do
         rc = hangup( 1 )
         rc = DeleteFile( semActive )
         rc = SysSleep( 5 );
         exit 0
      end
   end
end
else if Option = 'SEND' then do
   /*
    *  transmit messages (news & mail)
    */
   text = SetFlag(flgXmt)
   if text = '' then do
      text = CheckCmd(soupSend)
      if text = '' then do
         text = SetWorkingDir(xmtDir)
         if text = '' then do
            rc = DoImport()
            text = '-- nothing done'
            if FileExists(soupXmtZip) then do
               say 'sending' soupXmtMsgs '(' || soupXmtZip || ')'
               '@' unzipProg
               '@' echo soupSend
               '@' soupSend
               text = 'msg(s) transmitted'
               if rc \= 0 then
                  text = soupSend 'returned with an error'
               rc = SysFileDelete( soupXmtZip )
               if FileExists(soupXmtMsgs) then do
                  say 'moving' soupXmtMsgs 'again into' soupXmtZip
                  '@' zipProgSend
               end
               rc = DoImport()
            end
         end
      end
      rc = SendFlag( flgXmt,text )
   end
end
else if Option = 'RCVMAIL' then
   text = DoReception( soupRcvMail,rcvMailDir,flgRcvMail )
else if Option = 'RCVNEWS' then
   text = DoReception( soupRcvNews,rcvNewsDir,flgRcvNews )
else if Option = 'RCVNEWS2' then
   text = DoReception( soupRcvNews2,rcvNews2Dir,flgRcvNews2 )
else
   call Help 'ill option:' Option

say Option || ':' text
if substr(text,1,1) \= "-" then do
   rc = SysSleep( 30 )   /* do not close window immediately */
   say 'goodbye'         /* last chance for ^S...           */
end
exit 0



CleanUp:  procedure expose StartTime hangupISP semActive wasConnected cmdName
State = arg(1)

if State = 'ok'  |  State = 'aborted' then do
   deltaTime = time('S') - startTime
   if deltaTime < 0 then
      deltaTime = deltaTime + 86400
   say CmdName 'took' deltaTime || 's'
      
   if \wasConnected then
      rc = Hangup(1)
   say State
   rc = DeleteFile(semActive)
   rc = SysSleep( 30 )
   if State = 'aborted' then
      exit 3
   exit 0
end
else do
   say CmdName 'already active'
   say '(if actually not, delete' semActive 'manually)'
   exit 3 
end

AbsExit:
exit 99



/********************************************************************************
 *
 *  Do a single reception procedure
 */
DoReception: procedure expose semImport importProg,
                              soupRcvZip soupRcvAreas soupRcvMsgs zipProg yarnHistory
cmd     = arg(1)
dir     = arg(2)
flgfile = arg(3)

text = SetFlag(flgfile)
if text = '' then do
   text = CheckCmd(cmd)
   if text = '' then do
      text = SetWorkingDir(dir)
      if text = '' then do
         rc = DoImport()
         '@' echo cmd
         '@' cmd
         text = 'ok'
         if rc \= 0 then
            text = cmd 'returned with an error'
         rc = SendFlag( flgfile,text )     /* hier kann es ein Problem geben, wenn eine zweite Instanz gestartet wurde */
         rc = DoImport()
      end
      else
         rc = SendFlag( flgfile,text )
   end
   else
      rc = SendFlag( flgfile,text )
end
return text



/********************************************************************************
 *
 *  Import the received data to the Yarn database
 *  - on entry the current subdir contains the areas, *.msg or zip-files
 *  - sets semaphor to avoid simultaneous imports
 *
 */
DoImport: procedure expose semImport importProg soupRcvZip ,
                           soupRcvAreas soupRcvMsgs zipProg yarnHistory

/*
 *  pack the files into the soupRcvZip
 */
if \FileExists(soupRcvZip) then do
   if FileExists(soupRcvAreas)  |  FileExists(soupRcvMsgs) then
      '@' zipProg
end

/*
 *  wait until semaphor is free
 */
First = 1
do forever
   if \FileExists(soupRcvZip) then do
      if \First then
         say soupRcvZip 'vanished (was another instance active?)'
      return 0
   end
   if stream(semImport,'c','open write') = 'READY:' then
      leave
   if First then do
      say 'import locked, waiting for access'
      First = 0
   end
   rc = SysSleep( 5 )
end

/*
 *  repeat import until soupRcvZip has been successfully read...
 */
do forever
   if stream(yarnHistory,'c','open write') = 'READY:' then do
      rc = stream(yarnHistory,'c','close')
      '@' importProg soupRcvZip
      if \FileExists(soupRcvZip) then
         leave
      if stream(soupRcvZip,'c','query size') = 0 then do   /*** should not happen ***/
         rc = SysFileDelete( soupRcvZip )
         leave
      end
   end
   if First then do
      say 'import locked, waiting for access'
      First = 0
   end
   rc = SysSleep( 5 )
end
rc = DeleteFile( semImport )
return 1



/********************************************************************************
 *
 *  set the working directory
 *  if operation fails, return value is an error message, 
 *  otherwise empty string is returned
 */
SetWorkingDir: procedure
dir = arg(1)

if translate(directory(dir)) = translate(dir) then do
   say 'working directory is' dir
   return ''
end
return 'cannot change to' dir



/********************************************************************************
 *
 *  Check flag file
 */
CheckFlag: procedure
file = arg(1)
msg  = arg(2)

if FileExists(file) then do
   len = stream(file,'c','query size')
   if len \= 0 then do
      txt = linein( file )
      if txt \= '' then do
         say msg "'" || txt || "'"
         rc = DeleteFile( file )
/***test***/
         if rc \= 0 then
            say 'cannot delete' file || ', rc=' || rc 'len:' len
         return 1
      end
   end
end
return 0



SetFlag: procedure
flgfile = arg(1)

/*
 *  there is a very small case, when this could fail:
 *  another instance starts & finishs during the '|'
 */
if FileExists(flgfile)  |  stream(flgfile,'c','open write') \= 'READY:' then
   return '-- blocked'
else do
   len = stream(flgfile,'c','query size');
   if len \= 0 then                    /***test***/
      say flgfile 'has length' len
end
return ''



SendFlag: procedure
file = arg(1)
msg  = arg(2)

rc = lineout( file,msg )
rc = lineout( file )
return 1



CheckCmd: procedure
cmd = arg(1)

if cmd = '' then
   return '-- disabled'
return ''



DeleteFile: procedure
f = arg(1)

rc = stream( f,'c','close' )
rc = SysFileDelete( f )
return rc



/********************************************************************************
 *
 *  Connect to ISP, usually calls 'loginisp.cmd'
 *  If you have an auto dialer, then the pinging should be enough to start
 *  connecting.  'loginisp.cmd' can thus be an empty script...
 *
 *  Thanx to scgf@netcomuk.co.uk (Phillip Deackes) for the idea
 */
Connect: procedure expose hangupISP connectISP pingHost DEBUG CompCmdName startProc

/*
if DEBUG then do
   return 1
end
*/

do i = 1 to 10
   '@' pingHost
   if rc = 0 then
      return 1
end

say 'connecting to ISP'
'@' startProc '"YarnIo CHECKISP"' CompCmdName 'CHECKISP'

do i = 1 to 10
   '@' connectISP
   do j = 1 to 60
      rc = SysSleep(1)
      '@' pingHost
      if rc = 0 then
         return 0
   end j
   rc = Hangup( 0 )
   rc = SysSleep( 5 )
end i
say '*****************************'
say '*** impossible to connect ***'
say '*** check your equipment  ***'
say '*****************************'
say 'aborted...'
exit 3
return 0



/********************************************************************************
 *
 *  Hangup connection to your ISP, usually calls 'logoutisp.cmd'
 *  The called script/progam could do whatever you want, e.g. if you have
 *  established other connections the 'logoutisp.cmd' could reject the hangup etc...
 */
Hangup: procedure expose hangupISP
sayMsg = arg(1)

if sayMsg then
   say 'hanging up connection'
'@' hangupISP
return 1



FileExists: procedure
filemask = arg(1)
rc = SysFileTree( filemask,res,F,, )
return res.0 \= 0



Help: procedure expose CmdName
msg  = arg(1)

say 'failed:' msg
say ''
say CmdName '0.60 rg011096'
say CmdName 'receives and transmits news/mail to Yarn via VSoup'
say ''
say 'options:'
say '-SEND, -RCVMAIL, -RCVNEWS, -RCVNEWS2:  disable the corresponding service'
say 'no options given:                      perform all services'
exit( 3 )
